function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? Object(arguments[i]) : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys.push.apply(ownKeys, Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
import * as React from 'react';
import escape from 'lodash/escape';
import markdown from '@cloudflare/util-markdown';
import { I18n } from './i18nContext';
var tagRegex = /<(\d+)>(.*)<\/\1>|<(\d+)\/>/;
var newLineRegex = /(?:\r\n|\r|\n)/g;
function createElement(index, value, testId) {
  return /*#__PURE__*/React.createElement("span", {
    key: index,
    "data-testid": testId,
    dangerouslySetInnerHTML: {
      __html: value
    }
  });
}

/**
 * `formatElements` - parse string and return tree of react elements
 *
 * `value` is string to be formatted with <0>Paired<0/> or <0/> (unpaired)
 * placeholders. `Components` is a array of react Components which indexes
 * correspond to element indexes in formatted string
 */
function formatElements(value) {
  var Components = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var componentProps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var testId = arguments.length > 3 ? arguments[3] : undefined;
  // TODO: warn if there are any unprocessed elements
  // TODO: warn if element at `index` doesn't exist

  var treeIndex = 0;
  var parts = value.replace(newLineRegex, '').split(tagRegex);

  // If there are no inline elements, return value
  if (parts.length === 1) return [createElement(treeIndex, value, testId)];
  var tree = [];
  var before = parts.shift();
  if (before) {
    var element = createElement(treeIndex, before, testId);
    tree.push(element);
    if (typeof element !== 'string') {
      treeIndex++;
    }
  }
  for (var [index, children, after] of getElements(parts)) {
    if (!Components[index]) {
      window.console && console.warn("Missing Component for translation key: ".concat(value, ", index: ").concat(index, ". Fragment will be used."));
    }
    var Component = Components[index] || React.Fragment;
    var props = componentProps[index] || {};
    var _element = createElement(0, children);
    tree.push( /*#__PURE__*/React.createElement(Component, _extends({
      key: treeIndex
    }, props), _element));
    treeIndex++;
    if (after) {
      var _element2 = createElement(treeIndex, after);
      tree.push(_element2);
      if (typeof _element2 !== 'string') {
        treeIndex++;
      }
    }
  }
  return tree;
}
/*
 * `getElements` - return array of element indexes and element childrens
 *
 * `parts` is array of [pairedIndex, children, unpairedIndex, textAfter, ...]
 * where:
 * - `pairedIndex` is index of paired element (undef for unpaired)
 * - `children` are children of paired element (undef for unpaired)
 * - `unpairedIndex` is index of unpaired element (undef for paired)
 * - `textAfter` is string after all elements (empty string, if there's nothing)
 *
 * `parts` length is always multiply of 4
 *
 * Returns: Array<[elementIndex, children, after]>
 */
function getElements(parts) {
  if (!parts.length) return [];
  var [paired, children, unpaired, after] = parts.slice(0, 4);
  var element = [[parseInt(paired || unpaired), children || '', after]];
  return element.concat(getElements(parts.slice(4, parts.length)));
}
export function Trans(_ref) {
  var {
    id = '',
    smartCount,
    _,
    values,
    applyMarkdown,
    Components,
    componentProps,
    testId
  } = _ref;
  return /*#__PURE__*/React.createElement(I18n, null, translator => {
    // Ensure any interpolated values are escaped to prevent XSS and HTML injection
    if (values) {
      Object.keys(values).forEach(key => values[key] = escape(values[key]));
    }
    // Ensure any string props passed to React components are escaped to prevent potential XSS and HTML injection
    if (componentProps) {
      componentProps.forEach(props => {
        Object.keys(props).forEach(key => {
          if (typeof props[key] === 'string') {
            props[key] = escape(props[key]);
          }
        });
      });
    }
    var options = _objectSpread({
      smart_count: smartCount,
      _
    }, values);

    // Skip built-in markdown sanitization, as all inputs are already sanitized above
    // Having another round of sanitization here would strip all <0 /> component interpolation tags
    var text = applyMarkdown ? markdown(translator.t(id.toString(), options), {
      __dangerouslyDontSanitizeMarkdown: true
    }) : translator.t(id.toString(), options);
    var elements = formatElements(text, Components, componentProps, testId);
    return elements.length > 1 ? /*#__PURE__*/React.createElement(React.Fragment, null, elements) : elements[0];
  });
}
export default Trans;